//
// Created by dell on 24-9-26.
//

#include "mlx90614.h"
#include "gd32e23x.h"
#include "systick.h"
#include <stdio.h>

void i2c_config(void)
{
    rcu_periph_clock_enable(RCU_IR_GPIO);
    rcu_periph_clock_enable(RCU_I2C);

    gpio_af_set(I2C_SCL_PORT, I2C_GPIO_AF, I2C_SCL_PIN);
    gpio_af_set(I2C_SDA_PORT, I2C_GPIO_AF, I2C_SDA_PIN);

    gpio_mode_set(I2C_SCL_PORT, GPIO_MODE_AF, GPIO_PUPD_PULLUP, I2C_SCL_PIN);
    gpio_output_options_set(I2C_SCL_PORT, GPIO_OTYPE_OD, GPIO_OSPEED_50MHZ, I2C_SCL_PIN);

    gpio_mode_set(I2C_SDA_PORT, GPIO_MODE_AF, GPIO_PUPD_PULLUP, I2C_SDA_PIN);
    gpio_output_options_set(I2C_SDA_PORT, GPIO_OTYPE_OD, GPIO_OSPEED_50MHZ, I2C_SDA_PIN);

    i2c_clock_config(IR_I2C, I2C_SPEED, I2C_DTCY_2);

    i2c_mode_addr_config(IR_I2C, I2C_I2CMODE_ENABLE, I2C_ADDFORMAT_7BITS, SLAVE_ADDR);
    i2c_enable(IR_I2C);
    i2c_ack_config(IR_I2C, I2C_ACK_ENABLE);
}

uint32_t read_ir_mlx90614(void)
{
    uint8_t Data[3] = {0};
    uint32_t inttemp_ir = 0;
    uint16_t timeout = 0;

    // 发送起始信号
    i2c_start_on_bus(IR_I2C);
    while(i2c_flag_get(IR_I2C, I2C_FLAG_I2CBSY) && (timeout < I2C_TIME_OUT)) {
        timeout++;
    }
    if (timeout >= I2C_TIME_OUT) {
        return -41001; // 超时返回错误
    }

    // 发送从机地址和写操作位
    i2c_master_addressing(IR_I2C, SLAVE_ADDR, I2C_TRANSMITTER);
    while(!i2c_flag_get(IR_I2C, I2C_FLAG_ADDSEND) && (timeout < I2C_TIME_OUT)) {
        timeout++;
    }
    if (timeout >= I2C_TIME_OUT) {
        return -41002; // 超时返回错误
    }
    i2c_flag_clear(IR_I2C, I2C_FLAG_ADDSEND);

    // 发送寄存器地址
    i2c_data_transmit(IR_I2C, REG_ADDR_OBJ_TEMP);
    while(!i2c_flag_get(IR_I2C, I2C_FLAG_TBE) && (timeout < I2C_TIME_OUT)) {
        timeout++;
    }
    if (timeout >= I2C_TIME_OUT) {
        return -41003; // 超时返回错误
    }

    // 重复起始信号
    i2c_start_on_bus(IR_I2C);
    while(i2c_flag_get(IR_I2C, I2C_FLAG_I2CBSY) && (timeout < I2C_TIME_OUT)) {
        timeout++;
    }
    if (timeout >= I2C_TIME_OUT) {
        return -41004; // 超时返回错误
    }

    // 发送从机地址和读操作位
    i2c_master_addressing(IR_I2C, SLAVE_ADDR, I2C_RECEIVER);
    while(!i2c_flag_get(IR_I2C, I2C_FLAG_ADDSEND) && (timeout < I2C_TIME_OUT)) {
        timeout++;
    }
    if (timeout >= I2C_TIME_OUT) {
        return -41005; // 超时返回错误
    }
    i2c_flag_clear(IR_I2C, I2C_FLAG_ADDSEND);

    // 读取数据
    for (int i = 0; i < 3; i++) {
        while(!i2c_flag_get(IR_I2C, I2C_FLAG_RBNE) && (timeout < I2C_TIME_OUT)) {
            timeout++;
        }
        if (timeout >= I2C_TIME_OUT) {
            return -41006; // 超时返回错误
        }
        Data[i] = i2c_data_receive(IR_I2C);
        if (i < 2) {
            i2c_ack_config(IR_I2C, I2C_ACK_ENABLE);
        } else {
            i2c_ack_config(IR_I2C, I2C_ACK_DISABLE);
        }
        timeout = 0;
    }

    // 发送停止信号
    i2c_stop_on_bus(IR_I2C);

    // 计算温度
    inttemp_ir = ((uint16_t)(Data[1] << 8) | Data[0]) * 2 -27315;

    return inttemp_ir; // 返回温度值
}
